@@ -40,7 +40,6 @@ FileWatcher::FileWatcher(QObject *parent, const QVariantList &args)
 {
   setAspectRatioMode(Plasma::IgnoreAspectRatio);
   setHasConfigurationInterface(true);
-  setBackgroundHints(TranslucentBackground);
   resize(400, 200);
 }
 
@@ -66,6 +65,7 @@ void FileWatcher::init()
   KConfigGroup cg = config();
 
   QString path = cg.readEntry("path", QString());
+  setAssociatedApplicationUrls(KUrl(path));
   textItem->setDefaultTextColor(cg.readEntry("textColor", Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor)));
   textItem->setFont(cg.readEntry("font", Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont)));
 
@@ -101,6 +101,7 @@ void FileWatcher::constraintsEvent(Plasma::Constraints constraints)
 {
     if (constraints & Plasma::SizeConstraint){
         textItem->setSize((int) contentsRect().width(), (int) contentsRect().height());
+        textItem->setPos(contentsRect().topLeft());
         updateRows();
     }
 }
@@ -191,6 +192,7 @@ void FileWatcher::newData()
   }
 
   cursor.endEditBlock();
+  emit sizeHintChanged(Qt::PreferredSize);
 }
 
 void FileWatcher::createConfigurationInterface(KConfigDialog *parent)
@@ -244,8 +246,19 @@ void FileWatcher::configAccepted()
 
     textItem->update();
     loadFile(tmpPath);
+    setAssociatedApplicationUrls(KUrl(tmpPath));
 
     emit configNeedsSaving();
 }
 
+QSizeF FileWatcher::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
+{
+    QSizeF hint = QGraphicsWidget::sizeHint(which, constraint);
+    if (which == Qt::PreferredSize) {
+        hint.setHeight(qMax((qreal)200.0, textItem->document()->size().height()));
+    }
+
+    return hint;
+}
+
 #include "fileWatcher.moc"
