@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser/Library General Public License version 2,
@@ -21,80 +21,59 @@
 #define LANCELOT_GLOBAL_H
 
 #include <QColor>
-#include <QString>
-#include <QMap>
 #include <QList>
+#include <QMap>
+#include <QString>
 #include <QVariant>
+
 #include <KConfig>
 #include <KConfigGroup>
+
 #include <plasma/framesvg.h>
+#include <plasma/plasma.h>
 
 #include <lancelot/lancelot_export.h>
 
 namespace Lancelot
 {
 
-class Widget;
-class Instance;
+class Global;
 
 /**
- * Represents a group of widgets
+ * Represents a group of object
  *
  * @author Ivan Cukic
  *
- * This is not a visual grouping of widgets. For that, see QGraphicsItem,
+ * This is not a visual grouping of object. For that, see QGraphicsItem,
  * QGraphicsWidget etc. documentation.
  *
- * Lancelot::WidgetGroup provides a mechanism for grouping widgets
+ * Lancelot::Group provides a mechanism for grouping objects
  * semantically.
  *
  * For example, if you want to have a couple of buttons that should
  * change their appearances or behaviour simultaneously, you don't
- * need to change the properties for each widget, but only to notify
+ * need to change the properties for each object, but only to notify
  * the group object.
  *
  * One of the examples where this is heavily used is the theming
  * support in the %Lancelot menu application.
  *
  */
-class LANCELOT_EXPORT WidgetGroup : public QObject {
+class LANCELOT_EXPORT Group : public QObject {
 public:
 
     /**
      * This class contains the triplet od colors for
-     * three standard widget states - normal, disabled
+     * three standard object states - normal, disabled
      * and active
      */
     class ColorScheme {
     public:
+        ColorScheme();
+
         QColor normal, disabled, active;
     };
 
-    /**
-     * @param property the ID of the property
-     * @returns whether this group has the specified property set
-     */
-    bool hasProperty(int property) const;
-
-    /**
-     * @param property the ID of the property
-     * @returns the value of the specified property
-     */
-    QVariant property(int property) const;
-
-    /**
-     * Sets the value of the specified property
-     * @param property the ID of the property
-     * @param value value to assign to the property
-     */
-    void setProperty(int property, const QVariant & value);
-
-    /**
-     * Clears the value of the specified property
-     * @param property the ID of the property
-     */
-    void clearProperty(int property);
-
     /**
      * @param the name of the property
      * @returns whether this group has the specified property set
@@ -109,9 +88,11 @@ public:
 
     /**
      * Sets the value of the specified property
-     * @param the name of the property
+     * @param property the name of the property
+     * @param value value
+     * @param persistent whether the property should be preserved on group reload
      */
-    void setProperty(const QString & property, const QVariant & value);
+    void setProperty(const QString & property, const QVariant & value, bool persistent = true);
 
     /**
      * Clears the value of the specified property
@@ -140,12 +121,6 @@ public:
      */
     const ColorScheme * foregroundColor() const;
 
-    /**
-     * Notifies the widgets that belong to this group that the group
-     * is updated.
-     */
-    void notifyUpdated();
-
     /**
      * Loads the group properties from a configuration file.
      * @full if true, the group is reloaded even if it was already
@@ -154,96 +129,67 @@ public:
     void load(bool full = false);
 
     /**
-     * Adds widget to the group
-     * @param widget the widget to add to the group
+     * Adds object to the group
+     * @param object the object to add to the group
      */
-    void addWidget(Widget * widget);
+    void add(QObject * object);
 
     /**
-     * Removes the widget from the group
-     * @param widget the widget that should be removed from the group
-     * @param setDefaultGroup if set, the widget is moved to the default
-     *     group. If not, the widget will retain the pointer to the old
+     * @returns whether the groeup contains the specified object
+     * @param object the object
+     */
+    bool contains(QObject * object);
+
+    /**
+     * Removes the object from the group
+     * @param object the object that should be removed from the group
+     * @param setDefaultGroup if set, the object is moved to the default
+     *     group. If not, the object will retain the pointer to the old
      *     group, but will not receive any updates from it.
      */
-    void removeWidget(Widget * widget, bool setDefaultGroup = true);
+    void remove(QObject * object, bool setDefaultGroup = true);
 
     /**
      * @returns the name of the group
      */
     QString name() const;
 
-    /**
-     * @returns a pointer to the Lancelot::Instance to which this group
-     *     belongs.
-     */
-    Instance * instance();
+    // TODO: delete me after 4.3
+    void notifyUpdated() {}
 
 private:
     class Private;
     Private * const d;
 
-    WidgetGroup(Instance * instance, QString name);
-    virtual ~WidgetGroup();
+    Group(QString name);
+    virtual ~Group();
 
-    friend class Instance;
+    friend class Global;
 };
 
 /**
- * Instance object represents one instance of Lancelot-based process
+ * Global object represents one instance of Lancelot-based process
  * inside a main application.
  *
  * @author Ivan Cukic
  *
- * One application can have multiple Lancelot::Instances. For example,
+ * One application can have multiple Lancelot::Globals. For example,
  * when you add a couple of Lancelot applets to the Plasma desktop.
  *
- * Instances provide separate groups of widgets, separate themes etc.
+ * Globals provide separate groups of objects, separate themes etc.
  * Also provides some events inhibition (mostly related to geometry
  * processing) to speed up form creation and initialization and avoid
  * unnecessary setGeometry calls.
  */
-class LANCELOT_EXPORT Instance : public QObject {
+class LANCELOT_EXPORT Global : public QObject {
     Q_OBJECT
 public:
-    /**
-     * Creates a new Lancelot::Instance
-     */
-    Instance();
+    static Global * self();
 
     /**
-     * Destroys this Lancelot::Instance
+     * Destroys this Lancelot::Global
      */
-    virtual ~Instance();
-
-    /**
-     * @returns whether the instance is activated
-     */
-    bool isActivated();
-
-public Q_SLOTS:
-    /**
-     * Removes the event processing inhibitions and activates
-     * all widgets that belong to this instance.
-     */
-    void activateAll();
-
-    /**
-     * Activates the event processing inhibitions and deactivates
-     * all widgets that belong to this instance.
-     */
-    void deactivateAll();
-
-public:
-    /**
-     * Adds a widget to instance
-     */
-    void addWidget(Widget * widget);
-
-    /**
-     * Removes a widget from instance
-     */
-    void removeWidget(Widget * widget);
+    virtual ~Global();
 
     /**
      * @returns a pointer to the theme configuration file
@@ -259,46 +205,49 @@ public:
      * @param name group name
      * @returns a pointer to the group with specified name
      */
-    WidgetGroup * group(const QString & name);
+    Group * group(const QString & name);
+
+    /**
+     * @returns a group to which the object belongs
+     * @param object object
+     */
+    Group * groupForObject(QObject * object);
+
+    /**
+     * Sets the group for the object
+     */
+    void setGroupForObject(QObject * object, Group * group);
+
+    /**
+     * @returns whether a group with the specified name exists
+     */
+    bool groupExists(const QString & name) const;
 
     /**
      * @returns a pointer to the default group
      */
-    WidgetGroup * defaultGroup();
+    Group * defaultGroup();
 
     /**
-     * Active instance is the instance to which all currently created
-     * widgets are inserted into.
-     * \note
-     *   Note that only one instance can be active at a time
-     * @returns a pointer to the active instance
+     * Sets immutability
+     * @param immutability new immutability
      */
-    static Instance * activeInstance();
+    void setImmutability(const Plasma::ImmutabilityType immutable);
 
     /**
-     * Sets the active instance and locks the mutex for it
-     * @param instance new active instance
+     * @returns current immutability
      */
-    static void setActiveInstanceAndLock(Instance * instance);
+    Plasma::ImmutabilityType immutability() const;
 
-    /**
-     * Releases the lock on the active instance
-     */
-    static void releaseActiveInstanceLock();
-
-    // TODO: Replace the following finctions, and move them into the
-    // Instance itself.
-    /**
-     * @returns whether instance is in application
-     */
-    static bool hasApplication();
-
-    /**
-     * Sets whether instance is in application
-     */
-    static void setHasApplication(bool value);
+Q_SIGNALS:
+    void immutabilityChanged(const Plasma::ImmutabilityType immutable);
 
 private:
+    /**
+     * Creates a new Lancelot::Global
+     */
+    Global();
+
     class Private;
     Private * const d;
 };
