@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser/Library General Public License version 2,
@@ -17,25 +17,22 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#ifndef LANCELOT_PASSAGEWAY_VIEW_H_
-#define LANCELOT_PASSAGEWAY_VIEW_H_
+#ifndef LANCELOT_PASSAGEWAY_VIEW_H
+#define LANCELOT_PASSAGEWAY_VIEW_H
+
+#include <QtGui/QGraphicsLinearLayout>
+#include <QtGui/QIcon>
 
 #include <lancelot/lancelot.h>
 #include <lancelot/lancelot_export.h>
-
-#include <lancelot/widgets/Widget.h>
-#include <QIcon>
-
-#include <lancelot/widgets/Panel.h>
-#include <lancelot/widgets/ActionListView.h>
-#include <lancelot/widgets/PassagewayView.h>
 #include <lancelot/layouts/ColumnLayout.h>
-
-#include <lancelot/models/PassagewayViewModels.h>
-#include <lancelot/models/ActionListViewModels.h>
-
 #include <lancelot/layouts/NodeLayout.h>
-#include <QGraphicsLinearLayout>
+#include <lancelot/models/ActionListModel.h>
+#include <lancelot/models/ActionTreeModel.h>
+#include <lancelot/widgets/ActionListView.h>
+#include <lancelot/widgets/Panel.h>
+#include <lancelot/widgets/PassagewayView.h>
+#include <lancelot/widgets/Widget.h>
 
 namespace Lancelot
 {
@@ -49,36 +46,38 @@ class LANCELOT_EXPORT PassagewayView : public Lancelot::Panel
 {
     Q_OBJECT
 
-    L_WIDGET
-    L_INCLUDE(lancelot/widgets/PassagewayView.h)
+    Q_PROPERTY ( int activationMethod READ activationMethod WRITE setActivationMethod )
+
+    // @puck L_WIDGET
+    // @puck L_INCLUDE(lancelot/widgets/PassagewayView.h)
 
 public:
     PassagewayView(QGraphicsItem * parent = 0);
-    PassagewayView(PassagewayViewModel * entranceModel,
-            PassagewayViewModel * atlasModel = 0, QGraphicsItem * parent = 0);
+    explicit PassagewayView(ActionTreeModel * entranceModel,
+            ActionTreeModel * atlasModel = 0, QGraphicsItem * parent = 0);
     virtual ~PassagewayView();
 
     // Entrance
-    void setEntranceModel(PassagewayViewModel * model);
+    void setEntranceModel(ActionTreeModel * model);
     void setEntranceTitle(const QString & title);
     void setEntranceIcon(QIcon icon);
 
     // Atlas
-    void setAtlasModel(PassagewayViewModel * model);
+    void setAtlasModel(ActionTreeModel * model);
     void setAtlasTitle(const QString & title);
     void setAtlasIcon(QIcon icon);
 
-    void setActivationMethod(ActivationMethod value);
-    ActivationMethod activationMethod() const;
+    void setActivationMethod(int value);
+    int activationMethod() const;
 
     void setColumnLimit(int limit);
     void reset();
 
     void clearSelection();
 
-    L_Override virtual void setGroup(WidgetGroup * group = NULL);
-    L_Override virtual void groupUpdated();
-    L_Override virtual void keyPressEvent(QKeyEvent * event);
+    L_Override void setGroup(Group * group = NULL);
+    // L_Override void groupUpdated();
+    L_Override void keyPressEvent(QKeyEvent * event);
 
 protected Q_SLOTS:
     virtual void listItemActivated(int itemIndex, int listIndex = -1);
@@ -91,5 +90,5 @@ private:
 
 } // namespace Lancelot
 
-#endif /* LANCELOT_PASSAGEWAY_VIEW_H_ */
+#endif /* LANCELOT_PASSAGEWAY_VIEW_H */
 
