@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ *   Copyright (C) 2007 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser/Library General Public License version 2,
@@ -18,12 +18,14 @@
  */
 
 #include "ScrollPane.h"
-#include <QGraphicsSceneWheelEvent>
-
 #include "ScrollBar.h"
 #include "Widget.h"
+
+#include <Plasma/Animator>
+#include <QGraphicsSceneWheelEvent>
+#include <QDebug>
+
 #include <lancelot/layouts/FullBorderLayout.h>
-// #include <lancelot/layouts/FlipLayout.h>
 
 namespace Lancelot
 {
@@ -68,6 +70,8 @@ public:
           vertical(NULL), horizontal(NULL),
           flags(ScrollPane::ClipScrollable)
     {
+        qDebug() << "Plasma::Animator";
+        q->setAcceptTouchEvents(true);
     }
 
     void updateViewport()
@@ -115,7 +119,8 @@ ScrollPane::ScrollPane(QGraphicsItem * parent) //>
     d->layout->setContentsMargins(0, 0, 0, 0);
 
     setLayout(d->layout);
-    L_WIDGET_SET_INITIALIZED;
+
+    Plasma::Animator::self()->registerScrollingManager(this);
 } //<
 
 ScrollPane::~ScrollPane() //>
@@ -212,6 +217,12 @@ void ScrollPane::scrollableWidgetSizeUpdateNeeded() //>
         }
     }
 
+    if (hasVertical) {
+        Plasma::Animator::self()->registerScrollingManager(this);
+    } else {
+        Plasma::Animator::self()->unregisterScrollingManager(this);
+    }
+
     d->updateViewport();
 } //<
 
@@ -297,15 +308,6 @@ void ScrollPane::hoverLeaveEvent(QGraphicsSceneHoverEvent * event) //>
     }
 } //<
 
-void ScrollPane::wheelEvent(QGraphicsSceneWheelEvent * event) //>
-{
-    if (event->modifiers() & Qt::ControlModifier) {
-        d->horizontal->wheelEvent(event);
-    } else {
-        d->vertical->wheelEvent(event);
-    }
-} //<
-
 void ScrollPane::setFlip(Plasma::Flip flip) //>
 {
     if (flip & Plasma::HorizontalFlip) {
@@ -341,5 +343,28 @@ void ScrollPane::scrollTo(QRectF rect) //>
     }
 } //<
 
+QSizeF ScrollPane::contentsSize() const //>
+{
+    return d->widget->sizeFor(currentViewportSize());
+} //<
+
+void ScrollPane::setScrollPosition(const QPointF &position) //>
+{
+    scrollTo(QRectF(position, currentViewportSize()));
+} //<
+
+QPointF ScrollPane::scrollPosition() const //>
+{
+    if (!d || !d->horizontal || !d->vertical) {
+        return QPointF();
+    }
+    return QPointF(d->horizontal->value(), d->vertical->value());
+} //<
+
+QRectF ScrollPane::viewportGeometry() const //>
+{
+    return QRectF(QPointF(), currentViewportSize());
+} //<
+
 } // namespace Lancelot
 
