@@ -246,7 +246,9 @@ ComicProviderWrapper::ComicProviderWrapper( ComicProviderKross *parent )
       mKrossImage( 0 ),
       mPackage( 0 ),
       mRequests( 0 ),
-      mIdentifierSpecified( false )
+      mIdentifierSpecified( false ),
+      mIsLeftToRight( true ),
+      mIsTopToBottom( true )
 {
     QTimer::singleShot( 0, this, SLOT( init() ) );
 }
@@ -258,7 +260,7 @@ ComicProviderWrapper::~ComicProviderWrapper()
 
 void ComicProviderWrapper::init()
 {
-    const QString path = KStandardDirs::locate( "data", "plasma/comics/" + mProvider->pluginName() + "/" );
+    const QString path = KStandardDirs::locate( "data", "plasma/comics/" + mProvider->pluginName() + '/' );
     if ( !path.isEmpty() ) {
         mPackage = new Plasma::Package( path, ComicProviderKross::packageStructure() );
 
@@ -300,8 +302,8 @@ const QStringList& ComicProviderWrapper::extensions() const
         foreach( const QString &interpretername, Kross::Manager::self().interpreters() ) {
             info = Kross::Manager::self().interpreterInfo( interpretername );
             wildcards = info->wildcard();
-            wildcards.replace( "*", "" );
-            mExtensions << wildcards.split( " " );
+            wildcards.remove( '*' );
+            mExtensions << wildcards.split( ' ' );
         }
     }
     return mExtensions;
@@ -414,6 +416,26 @@ bool ComicProviderWrapper::identifierSpecified() const
     return mIdentifierSpecified;
 }
 
+bool ComicProviderWrapper::isLeftToRight() const
+{
+    return mIsLeftToRight;
+}
+
+void ComicProviderWrapper::setLeftToRight( bool ltr )
+{
+    mIsLeftToRight = ltr;
+}
+
+bool ComicProviderWrapper::isTopToBottom() const
+{
+    return mIsTopToBottom;
+}
+
+void ComicProviderWrapper::setTopToBottom( bool ttb )
+{
+    mIsTopToBottom = ttb;
+}
+
 QString ComicProviderWrapper::textCodec() const
 {
     return QString( mTextCodec );
@@ -678,6 +700,7 @@ void ComicProviderWrapper::finished() const
 {
     kDebug() << QString( "Author" ).leftJustified( 22, '.' ) << comicAuthor();
     kDebug() << QString( "Website URL" ).leftJustified( 22, '.' ) << mWebsiteUrl;
+    kDebug() << QString( "Shop URL" ).leftJustified( 22, '.' ) << mShopUrl;
     kDebug() << QString( "Title" ).leftJustified( 22, '.' ) << mTitle;
     kDebug() << QString( "Additional Text" ).leftJustified( 22, '.' ) << mAdditionalText;
     kDebug() << QString( "Identifier" ).leftJustified( 22, '.' ) << mIdentifier;
